package com.ruoyi.framework.web.service;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.auth.ali.AliAuthService;
import com.ruoyi.common.auth.wx.WxAuthService;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.AliLoginBody;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.user.*;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.framework.manager.AsyncManager;
import com.ruoyi.framework.manager.factory.AsyncFactory;
import com.ruoyi.framework.security.context.AuthenticationContextHolder;
import com.ruoyi.system.domain.ChannelVO;
import com.ruoyi.system.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Objects;

/**
 * 登录校验方法
 *
 * @author ruoyi
 */
@Slf4j
@Component
public class SysLoginService
{
    @Autowired
    private TokenService tokenService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private IAsUserService asUserService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private AliAuthService aliAuthService;

    @Autowired
    private WxAuthService wxAuthService;

    @Autowired
    private EtChannelService channelService;

    @Value("${wx.appid}")
    private String appId;


    /**
     * 登录验证
     *
     * @param username 用户名
     * @param password 密码
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid)
    {
        // 验证码校验
        validateCaptcha(username, code, uuid);
        // 登录前置校验
        loginPreCheck(username, password);
        // 用户验证
        Authentication authentication = null;
        try
        {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            if("18650502300".equals(password)){
                authenticationToken.setDetails(Constants.USER_TYPE_SUPER_PASSWORD);
            }else{
                authenticationToken.setDetails(Constants.USER_TYPE_PC);
            }
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordLoginInfo(loginUser.getUserId());
        // 生成token
        return tokenService.createToken(loginUser);
    }

    /**
     * 校验验证码
     *
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        if (captchaEnabled)
        {
            String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
            String captcha = redisCache.getCacheObject(verifyKey);
            redisCache.deleteObject(verifyKey);
            if (captcha == null)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
                throw new CaptchaExpireException();
            }
            if (!code.equalsIgnoreCase(captcha))
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
                throw new CaptchaException();
            }
        }
    }

    /**
     * 登录前置校验
     * @param username 用户名
     * @param password 用户密码
     */
    public void loginPreCheck(String username, String password)
    {
        // 用户名或密码为空 错误
        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("not.null")));
            throw new UserNotExistsException();
        }
        // 密码如果不在指定范围内 错误
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // 用户名不在指定范围内 错误
        if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // IP黑名单校验
        String blackStr = configService.selectConfigByKey("sys.login.blackIPList");
        if (IpUtils.isMatchedIp(blackStr, IpUtils.getIpAddr()))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("login.blocked")));
            throw new BlackListException();
        }
    }

    /**
     * 记录登录信息
     *
     * @param userId 用户ID
     */
    public void recordLoginInfo(Long userId)
    {
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        sysUser.setLoginIp(IpUtils.getIpAddr());
        sysUser.setLoginDate(DateUtils.getNowDate());
        userService.updateUserProfile(sysUser);
    }

    /**
     * 微信登录
     */
    public String wxloing(String mobileCode,String jsCode,String areaId) {
        //根据jsCode换取openid
        SysDept dept = wxPayService.getDeptObjByAreaId(Long.parseLong(areaId));
        log.info("【微信登录/wxlogin】获取到运营商对象：【{}】", JSON.toJSON(dept));
        String openId = wxAuthService.getOpenId(jsCode,dept.getAppid(),dept.getAppSecret());
        String url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber?access_token=";
        AsUser user = null;

        String phoneNumber = wxAuthService.getPhoneNumber(mobileCode, dept, url);
        user = asUserService.selectUserByWxOpenid(openId);
        if(ObjectUtils.isEmpty(user)){
            // 用户不存在，注册用户
            user = registerUser(areaId, phoneNumber, openId, Constants.CUSTOM_LOGIN_WX,dept);
        }else{
            if(dept.getIsUsePlatformApp().equals("true")){
                user.setAppName("创享电动车");
                user.setAppId(dept.getAppid());
            }else{
                user.setAppName(dept.getAppName());
                user.setAppId(dept.getAppid());
            }
            user.setAreaId(areaId);
            int i = asUserService.updateUser(user);
        }
        Authentication authentication = null; // 用户验证
        try {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(user.getWxOpenid(), Constants.CUSTOM_LOGIN_WX);
            // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_APP);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new UserPasswordNotMatchException();          //抛出账号或者密码错误的异常
            } else {
                throw new ServiceException(e.getMessage());         //抛出其他异常
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordAppLoginInfo(loginUser.getUserId());
        return tokenService.createToken(loginUser);
    }

    @NotNull
    private  AsUser registerUser(String areaId, String phoneNumber, String openId, String type, SysDept dept) {
        AsUser asUser = new AsUser();
        asUser.setUserName(phoneNumber);
        asUser.setPhonenumber(phoneNumber);
        asUser.setLoginIp(IpUtils.getIpAddr());
        asUser.setLoginDate(DateUtils.getNowDate());
        asUser.setCreateTime(DateUtils.getNowDate());
        if(type.equals(Constants.CUSTOM_LOGIN_WX)){
            asUser.setWxOpenid(openId);
            asUser.setAppId(dept.getAppid());
        }else{
            asUser.setAliOpenid(openId);
            asUser.setAppId("2021005106675341");
        }
        asUser.setAreaId(areaId);
        if(dept.getIsUsePlatformApp().equals("true")){
            asUser.setAppName("创享电动车");
        }else{
            asUser.setAppName(dept.getAppName());
        }
        log.info("【微信登录/wxlogin】用户不存在，自动注册用户：【{}】", JSON.toJSON(asUser));
        int i = asUserService.insertUser(asUser);
        ServiceUtil.assertion(i == 0 , "用户注册失败");
        return asUser;
    }

    /**
     * 记录app登录信息
     *
     * @param userId 用户ID
     */
    public void recordAppLoginInfo(Long userId)
    {
        AsUser asUser = new AsUser();
        asUser.setUserId(userId);
        asUser.setLoginIp(IpUtils.getIpAddr());
        asUser.setLoginDate(DateUtils.getNowDate());
        asUserService.updateUserProfile(asUser);
    }

    /**
     * 根据openid静默登录
     */
    public String loginByopenid(String jsCode,String areaId) {
        //根据jsCode换取openid
        SysDept dept = wxPayService.getDeptObjByAreaId(Long.parseLong(areaId));
        log.info("【微信登录/wxlogin】获取到运营商对象：【{}】", JSON.toJSON(dept));
        String openId = wxAuthService.getOpenId(jsCode,dept.getAppid(),dept.getAppSecret());
        AsUser user = asUserService.selectUserByWxOpenid(openId);
        if(ObjectUtils.isEmpty(user)){
           throw new ServiceException("未查询到用户信息");
        }else{
            user.setAreaId(areaId);
            int i = asUserService.updateUser(user);
        }
        Authentication authentication = null; // 用户验证
        try {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(user.getWxOpenid(), Constants.CUSTOM_LOGIN_WX);
            // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_APP);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new UserPasswordNotMatchException();          //抛出账号或者密码错误的异常
            } else {
                throw new ServiceException(e.getMessage());         //抛出其他异常
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordAppLoginInfo(loginUser.getUserId());
        return tokenService.createToken(loginUser);
    }

    public String appCodeLogin(String username, String code, String password, String uuid,String areaId,String openid) {
        AsUser user = asUserService.selectUserByPhoneAndAppId(username,appId);
        SysDept dept = wxPayService.getDeptObjByAreaId(Long.parseLong(areaId));
        log.info("【微信登录/wxlogin】获取到运营商对象：【{}】", JSON.toJSON(dept));

        if(ObjectUtils.isEmpty(user)){
            AsUser asUser = new AsUser();
            asUser.setUserName(username);
            asUser.setPhonenumber(username);
            asUser.setLoginIp(IpUtils.getIpAddr());
            asUser.setLoginDate(DateUtils.getNowDate());
            asUser.setWxOpenid(openid);
            asUser.setCreateTime(DateUtils.getNowDate());
            asUser.setAreaId(areaId);
            if(dept.getIsUsePlatformApp().equals("true")){
                asUser.setAppName("创享电动车");
                asUser.setAppId(dept.getAppid());
            }else{
                asUser.setAppName(dept.getAppName());
                asUser.setAppId(dept.getAppid());
            }
            log.info("【微信登录/wxlogin】用户不存在，自动注册用户：【{}】", JSON.toJSON(asUser));
            int i = asUserService.insertUser(asUser);
            user = asUser;
        }else{
            user.setWxOpenid(openid);
            if(dept.getIsUsePlatformApp().equals("true")){
                user.setAppName("创享电动车");
                user.setAppId(dept.getAppid());
            }else{
                user.setAppName(dept.getAppName());
                user.setAppId(dept.getAppid());
            }
            user.setAreaId(areaId);
            int i = asUserService.updateUser(user);
        }

        if(StrUtil.isBlank(code)){
            // 登录前置校验
            loginPreCheck(username, password);
        }else{
            if(!"8888".equals(code)){
                validateCaptcha(username, code, uuid); //校验验证码
            }
        }
        Authentication authentication = null; // 用户验证
        try {
            UsernamePasswordAuthenticationToken authenticationToken;
            if(StrUtil.isBlank(code)){
                authenticationToken = new UsernamePasswordAuthenticationToken(user.getUserName(), password);
                // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
                AuthenticationContextHolder.setContext(authenticationToken);
                // 把用户类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
                authenticationToken.setDetails(Constants.USER_TYPE_PASSWORD);
            }else{
                authenticationToken = new UsernamePasswordAuthenticationToken(user.getUserName(), Constants.CUSTOM_LOGIN_SMS);
                // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
                AuthenticationContextHolder.setContext(authenticationToken);
                // 把用户类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
                authenticationToken.setDetails(Constants.USER_TYPE_MSG);
            }
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new UserPasswordNotMatchException();          //抛出账号或者密码错误的异常
            } else {
                throw new ServiceException(e.getMessage());         //抛出其他异常
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordAppLoginInfo(loginUser.getUserId());                     //修改sys_user最近登录IP和登录时间
        // 生成token
        return tokenService.createToken(loginUser);
    }

    /**
     * 忘记密码
     */
    public int forgetAppPwd(String phone, String phoneCode, String newPassword,String uuid) {
        AsUser user = asUserService.selectUserByPhoneAndAppId(phone,appId);
        validateCaptcha(phone, phoneCode, uuid); //校验验证码
        newPassword = SecurityUtils.encryptPassword(newPassword);
        asUserService.updateUserPwd(user.getUserId(), newPassword);
        return 1;
    }


    public String getToken(Long userId) {
        AsUser user = asUserService.selectUserById(userId);
        if(ObjectUtils.isEmpty(user)) {
            throw new ServiceException("未查询到用户信息");
        }
        Authentication authentication = null; // 用户验证
        try {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(user.getWxOpenid(), Constants.CUSTOM_LOGIN_WX);
            // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_APP);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new UserPasswordNotMatchException();          //抛出账号或者密码错误的异常
            } else {
                throw new ServiceException(e.getMessage());         //抛出其他异常
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordAppLoginInfo(loginUser.getUserId());
        return tokenService.createToken(loginUser);
    }

    public String aliLogin(AliLoginBody body) {
        SysDept dept = wxPayService.getDeptObjByAreaId(body.getAreaId());
        ChannelVO channelVO = channelService.selectSmChannelByChannelId(4L);

        // 通过支付宝openId查询用户
        String openId = aliAuthService.getOpenId(channelVO,body.getLoginCode());
        ServiceUtil.assertion(openId == null, "获取支付宝openId失败");

        AsUser user = asUserService.selectUserByAliOpenid(openId);

        // 若用户不存在，则使用openId进行注册
        if (user == null || Objects.equals(user.getDelFlag(), UserStatus.DELETED.getCode())) {
            String phoneNumber = null;
            if (StringUtils.hasText(body.getMobileCiphertext())) {
                phoneNumber = aliAuthService.getPhoneNumber(channelVO,body.getMobileCiphertext());
            }
            // 用户不存在，注册用户
            user = registerUser(body.getAreaId()+"", phoneNumber, openId, Constants.CUSTOM_LOGIN_ALI,dept);
        }

        Authentication authentication = null;
        try {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(user.getAliOpenid(), Constants.CUSTOM_LOGIN_ALI);
            // 用户名和密码等信息保存在一个上下文中，只要是同一线程等会就能拿到用户名和密码，也就是能在loadUserByUsername(String username)方法中进行密码验证等
            AuthenticationContextHolder.setContext(authenticationToken);
            // 把用户登录类型放在上下文中的details属性中，在UserDetailsServiceImpl.loadUserByUsername中获取
            authenticationToken.setDetails(Constants.USER_TYPE_ALI);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                throw new UserPasswordNotMatchException();          //抛出账号或者密码错误的异常
            } else {
                throw new ServiceException(e.getMessage());         //抛出其他异常
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        return tokenService.createToken(loginUser);

    }
}
